% Linear cavity parameters
clear
close all
clc

lambda=1064e-9; % Wavelength
R1 = Inf;%10e-2; % Radius of cuvature of left mirror
R2 = 50e-2; % Radius of curvature of right mirror
 L = 10e-2; % Length of cavity
MirrorThickness=1e-2;

g1=1-L/R1;
g2=1-L/R2;

if g1*g2 >=0 && g1*g2 <= 1
    display('Stable')
else
    display('Unstable')
end

z1=g2*(1-g1)*L/(g1+g2-2*g1*g2); % Distance from waist to left mirror
z2=g1*(1-g2)*L/(g1+g2-2*g1*g2); % Distance from waist to right mirror

waist=sqrt(L*lambda/pi*sqrt(g1*g2*(1-g1*g2)/(g1+g2-2*g1*g2)^2)); % Beam waist in cavity
LeftMirrorWaist=sqrt(L*lambda/pi*sqrt(g2/(g1*(1-g1*g2)))); % Beam radius at left mirror
RightMirrorWaist=sqrt(L*lambda/pi*sqrt(g1/(g2*(1-g1*g2)))); % Beam radius at right mirror

CavityWaist=waist;

display(['g1 = ',num2str(g1)]);
display(['g2 = ',num2str(g2)]);
display(['z1 = ',num2str(z1*1e2),' cm']);
display(['z2 = ',num2str(z2*1e2),' cm']);
display(['w0 = ',num2str(waist*1e6),' microns']);
display(['w1 = ',num2str(LeftMirrorWaist*1e6),' microns']);
display(['w2 = ',num2str(RightMirrorWaist*1e6),' microns']);


%% clear
close all
% clc
cd('C:\Users\Admin\Desktop\Gaussian Cavity Beams')
set(0,'DefaultFigureWindowStyle','docked')

% This labels the figure that all of the beam profiles will be plotted on
InputBeam=figure;

lambda=1055e-9; % Wavelength of input beam
N=2000; % Number of points to plot in each section

% Calculation of beam parameter from known beam radii
InCon=0.0254; % This is the conversion from inches to meters

% Known beam positions and radii
% The columns are Measured [Position in inches, Dx, Dy].  Dx and Dy are
% diameter in the x and y directions.
% We will convert to meters and waist in the next step
Measured=[ 2, 1492, 1497;...
           5, 1455, 1433;...
           8, 1460, 1425
           12, 1455, 1415];%;...
           
%            8, 1525, 1437;...
%           10, 1530, 1444;...
%           12, 1550, 1450;...
%           14, 1558, 1460;...
%           16, 1574, 1466];

  Measured(:,1)=Measured(:,1)*InCon; % This converts the inches to meters
Measured(:,2:3)=Measured(:,2:3)/2*1e-6; % This converts the diameter in um's to radius in meters

MeasuredDimension='x';

if MeasuredDimension=='x'
    MeasuredDimension=2;
else MeasuredDimension=3;
end

% Now pick which two points along your measured beam to calculate the
% gaussian beam from
MeasuredPoint1=1;
MeasuredPoint2=4;
w1=Measured(MeasuredPoint1,MeasuredDimension);
x1=Measured(MeasuredPoint1,1);

w2=Measured(MeasuredPoint2,MeasuredDimension);
x2=Measured(MeasuredPoint2,1);

% These two lines solve the gaussian beam equations for the waist and
% position of the waist that will be used later
syms w0 z0
[w0,z0]=solve(w1^2==w0^2*(1+(lambda*(z0+x1)/(pi*w0^2))^2),w2^2==w0^2*(1+(lambda*(z0+x2)/(pi*w0^2))^2));

z0=double(z0)
w0=double(w0)

% This plots the measured beam spot sizes and postions
figure(InputBeam)
plot(Measured(:,1),Measured(:,MeasuredDimension),'xr')
hold all
plot(Measured(:,1),-Measured(:,MeasuredDimension),'xr')
hold all


% Initial conditions
z0=z0(1); % Position of known waist and radius
w0=w0(1); % Beam radius at z=0

R=z0*(1+(pi*w0^2/(lambda*z0))^2);
w=w0*sqrt(1+(lambda*z0/(pi*w0^2))^2);

% R=inf;
% w=1e-3;

zi=0;         % Initial position, restart it at zero
n=1;          % Initial index of refraction
% Initial q value of input beam
q1=(1/R)/(1/R^2+(lambda/(pi*n*w^2))^2)+1i*lambda/(pi*n*w^2)/(1/R^2+(lambda/(pi*n*w^2))^2);

% f= 300 mm -> D1=38 cm
% f= 250 mm -> D1=25 cm

Total_Distance=65e-2;
D1=10e-2; % Try to stay around 24 cm so there is room on the table
D2=Total_Distance-D1; % Must be after 69 cm to keep on the bread board
D3=Total_Distance-D1-D2;

Focal_Length1=750e-3;
Focal_Length2=250e-3;

%---------------------------------
%-----------TRANSLATION-----------
%---------------------------------
n=1;     % Index of refraction
d=D1; % Distance translated
[q,~,~,z]=GBeam_L(q1,lambda,n,d,N,zi,'PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

%---------------------------------
%------------LENS-----------------
%---------------------------------
n=1;     % Index of refraction of medium lens is in
f=Focal_Length1; % Focal length of lens
Angle=0; % Angle lens is tilted (radians)
[q]=GBeam_Lens(q(end),lambda,n,f,Angle,z,'Tan','PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

%---------------------------------
%-----------TRANSLATION-----------
%---------------------------------
n=1;     % Index of refraction
d=D2; % Distance translated
[q,~,~,z]=GBeam_L(q,lambda,n,d,N,z,'PlotElement','On','PlotHandle',InputBeam);
%---------------------------------
%{
%---------------------------------
%------------LENS-----------------
%---------------------------------
n=1;     % Index of refraction of medium lens is in
f=Focal_Length2; % Focal length of lens
Angle=0; % Angle lens is tilted (radians)
[q]=GBeam_Lens(q(end),lambda,n,f,Angle,z,'Tan','PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

%---------------------------------
%-----------TRANSLATION-----------
%---------------------------------
n=1;     % Index of refraction
d=D3; % Distance translated
[q,~,~,z]=GBeam_L(q(end),lambda,n,d,N,z(end),'PlotElement','On','PlotHandle',InputBeam);
%---------------------------------
%}
% Input coupler side 1
%---------------------------------
%------------INTERFACE------------
%---------------------------------
n1=1;   % Index before interface
n2=1.5; % Index after interface
RoC=Inf;  % Radius of curvature of surface
Angle=0; 
[q]=GBeam_Inter(q(end),lambda,n1,n2,RoC,Angle,z(end),'Tan','PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

%---------------------------------
%-----------TRANSLATION-----------
%---------------------------------
n=1.5;  % Index of refraction
d=MirrorThickness; % Distance translated
[q,~,~,z]=GBeam_L(q(end),lambda,n,d,N,z(end),'PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

% Input coupler side 2
%---------------------------------
%------------INTERFACE------------
%---------------------------------
n1=1.5; % Index of refraction before interface
n2=1;   % Index of refraction after interface
RoC=R1; % Radius of curvature from above
Angle=0;
[q]=GBeam_Inter(q(end),lambda,n1,n2,RoC,Angle,z(end),'Tan','PlotElement','On','PlotHandle',InputBeam);
%---------------------------------
zInput=z(end);

%---------------------------------
%-----------TRANSLATION-----------
%---------------------------------
n=1;    % Index of refraction
d=L;      % Distance translated depends on curved mirror separation from BowTieCavityProgram
[q,wMin,~,z]=GBeam_L(q(end),lambda,n,d,N,z(end),'PlotElement','On','PlotHandle',InputBeam);
%---------------------------------
[Waist,MinIndex]=min(wMin);
display(['Waist in cavity = ',num2str(Waist(1)*1e6),' microns'])

% First interface of output coupler
%---------------------------------
%------------INTERFACE------------
%---------------------------------
n1=1;   % Index of refraction before interface
n2=1.5; % Index of refraction after interface
RoC=R2;  % Radius of curvature
Angle=0; 
[q]=GBeam_Inter(q(end),lambda,n1,n2,RoC,Angle,z(end),'Tan','PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

%---------------------------------
%-----------TRANSLATION-----------
%---------------------------------
n=1.5;    % Index of refraction
d=1e-2;      % Distance translated depends on curved mirror separation from BowTieCavityProgram
[q,~,~,z]=GBeam_L(q(end),lambda,n,d,N,z(end),'PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

%---------------------------------
%------------INTERFACE------------
%---------------------------------
n1=1.5; % Index of refraction before interface
n2=1;   % Index of refraction after interface
RoC=R1; % Radius of curvature from above
Angle=0;
[q]=GBeam_Inter(q(end),lambda,n1,n2,RoC,Angle,z(end),'Tan','PlotElement','On','PlotHandle',InputBeam);
%---------------------------------
%-----------TRANSLATION-----------
%---------------------------------
n=1;    % Index of refraction
d=1e-2; 
[q,~,~,z]=GBeam_L(q(end),lambda,n,d,N,z(end),'PlotElement','On','PlotHandle',InputBeam);
%---------------------------------

% This finds the beam profile of the cavity
N=100; % Number of points in cavity mode
CavityZ1=0:L/N:(L-z2);
CavityZ2=0:L/N:(L-z1);
CavityMode1=CavityWaist*sqrt(1+(lambda.*CavityZ1./(pi.*CavityWaist.^2)).^2);
CavityMode2=CavityWaist*sqrt(1+(lambda.*CavityZ2./(pi.*CavityWaist.^2)).^2);

% This plots the cavity mode
plot(zInput+z1-CavityZ1,CavityMode1,'-k')
hold all
plot(zInput+z1-CavityZ1,-CavityMode1,'-k')
hold all
plot(zInput+z1+CavityZ2,CavityMode2,'-k')
hold all
plot(zInput+z1+CavityZ2,-CavityMode2,'-k')
% This plots the cavity waist
line([zInput+z1,zInput+z1],[CavityWaist,-CavityWaist],'Color','k','LineStyle','--')


zoom on


